package org.foo.bar.mysql;

import static org.osgi.service.jdbc.DataSourceFactory.JDBC_PASSWORD;
import static org.osgi.service.jdbc.DataSourceFactory.JDBC_URL;
import static org.osgi.service.jdbc.DataSourceFactory.JDBC_USER;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

/**
 * An abbreviated/simplified DataSource impl that takes a URL from the client
 * and just returns a thin data source wrapper around the basic JDBC driver.
 */
final class UrlBasedDriverDataSource implements javax.sql.DataSource {

    Driver     driver;
    Properties properties = null;
    String     url        = null;

    /**
     * @param properties
     *            The properties to use for operations on the driver
     * @param embedded
     *            Whether to wrap an embedded or a client driver
     */
    public UrlBasedDriverDataSource(final Properties properties) throws Exception {
        this.driver = new com.mysql.jdbc.Driver();
        this.properties = (Properties) properties.clone();
        this.url = properties.getProperty(JDBC_URL);
    }

    @Override
    public Connection getConnection() throws java.sql.SQLException {
        return driver.connect(url, properties);
    }

    @Override
    public Connection getConnection(final String user, final String password) throws java.sql.SQLException {
        Properties localProps = (Properties) properties.clone();
        localProps.put(JDBC_USER, user);
        localProps.put(JDBC_PASSWORD, password);
        return driver.connect(url, localProps);
    }

    @Override
    public boolean isWrapperFor(final Class<?> cls) {
        return (cls == com.mysql.jdbc.Driver.class);
    }

    @SuppressWarnings("unchecked")
    @Override
    public <T> T unwrap(final Class<T> cls) {
        return (isWrapperFor(cls)) ? (T) driver : null;
    }

    @Override
    public PrintWriter getLogWriter() throws SQLException {
        return DriverManager.getLogWriter();
    }

    @Override
    public int getLoginTimeout() throws SQLException {
        return DriverManager.getLoginTimeout();
    }

    // Don't support setting log writer or timeout

    @Override
    public void setLogWriter(final PrintWriter writer) throws SQLException {
        throw new SQLException("Can't set Log Writer on URL data source");
    }

    @Override
    public void setLoginTimeout(final int timeout) throws SQLException {
        throw new SQLException("Can't set Login Timeout on URL data source");
    }
}